@@ -381,30 +381,48 @@ class RemoteDataset(Dataset):
 
 
 class IRDSDataset(Dataset):
-    def __init__(self, irds_id):
+    def __init__(self, irds_id, defer_load=False):
         self._irds_id = irds_id
-        self._irds_ref = ir_datasets.load(self._irds_id)
+        self._irds_ref = None if defer_load else ir_datasets.load(self._irds_id)
 
     def irds_ref(self):
+        if self._irds_ref is None:
+            self._irds_ref = ir_datasets.load(self._irds_id)
         return self._irds_ref
 
     def get_corpus(self):
         raise NotImplementedError("IRDSDataset doesn't support get_corpus; use get_corpus_iter instead. If you "
                                   "are indexing, get_corpus_iter should be used in conjunction with IterDictIndexer.")
 
-    def get_corpus_iter(self, verbose=True):
+    def get_corpus_iter(self, verbose=True, start=0, count=None):
         ds = self.irds_ref()
         assert ds.has_docs(), f"{self._irds_id} doesn't support get_corpus_iter"
         it = ds.docs_iter()
+        total = ds.docs_count()
+
+        # use slicing if requested
+        if start > 0 or count is not None:
+            if count is not None:
+                it = it[start:start+count]
+                total = count
+            else:
+                it = it[start:]
+                total -= start
+        
+        # tqdm support
         if verbose:
-            it = tqdm(it, desc=f'{self._irds_id} documents', total=ds.docs_count())
+            it = tqdm(it, desc=f'{self._irds_id} documents', total=total)
+
+        # rewrite to follow pyterrier std
         def gen():
             for doc in it:
                 doc = doc._asdict()
                 # pyterrier uses "docno"
                 doc['docno'] = doc.pop('doc_id')
                 yield doc
-        return GeneratorLen(gen(), ds.docs_count())
+
+        # ensure we can provide accurate len
+        return GeneratorLen(gen(), total)
 
     def get_corpus_lang(self):
         ds = self.irds_ref()
@@ -429,8 +447,13 @@ class IRDSDataset(Dataset):
         df.rename(columns={"query_id": "qid"}, inplace=True) # pyterrier uses "qid"
 
         if variant is not None:
+            # Some datasets have a query field called "query". We need to remove it or
+            # we'll end up with multiple "query" columns, which will cause problems
+            # because many components are written assuming no columns have the same name.
+            if variant != 'query' and 'query' in df.columns:
+                df.drop(columns=['query'], axis=1, inplace=True)
             df.rename(columns={variant: "query"}, inplace=True) # user specified which version of the query they want
-            df.drop(df.columns.difference(['qid','query']), 1, inplace=True)
+            df.drop(columns=df.columns.difference(['qid','query']), axis=1, inplace=True)
         elif len(qcls._fields) == 2:
             # auto-rename single query field to "query" if there's only query_id and that field
             df.rename(columns={qcls._fields[1]: "query"}, inplace=True)
@@ -1056,7 +1079,7 @@ DATASET_MAP = {
 # ...
 import ir_datasets
 for ds_id in ir_datasets.registry:
-    DATASET_MAP[f'irds:{ds_id}'] = IRDSDataset(ds_id)
+    DATASET_MAP[f'irds:{ds_id}'] = IRDSDataset(ds_id, defer_load=True)
 
 # "trec-deep-learning-docs"
 #DATASET_MAP['msmarco_document'] = DATASET_MAP["trec-deep-learning-docs"]
