@@ -18,6 +18,7 @@ from warnings import warn
 from deprecated import deprecated
 from collections import deque
 from typing import List, Dict, Union, Any
+import more_itertools
 
 StringReader = None
 HashMap = None
@@ -584,7 +585,11 @@ class DFIndexUtils:
 
     @staticmethod
     def get_column_lengths(df):
-        return dict([(v, df[v].apply(lambda r: len(str(r)) if r!=None else 0).max())for v in df.columns.values])
+        import math
+        meta2len = dict([(v, df[v].apply(lambda r: len(str(r)) if r!=None else 0).max())for v in df.columns.values])
+        # nan values can arise if df is empty. Here we take a metalength of 1 instead.
+        meta2len = {k : 1 if math.isnan(l) else l for k, l in meta2len.items()}
+        return meta2len
 
     @staticmethod
     def create_javaDocIterator(text, *args, **kwargs):
@@ -613,6 +618,8 @@ class DFIndexUtils:
             else:
                 raise ValueError("Keyword kwargs need to be of type pandas.Series, list or tuple")
 
+        if "docno" not in all_metadata:
+            raise ValueError('No docno column specified, while PyTerrier assumes a docno should exist. Found meta columns were %s' % str(list(all_metadata.keys())))
         # this method creates the documents as and when needed.
         def convertDoc(text_row, meta_column):
             if text_row is None:
@@ -807,13 +814,43 @@ class _BaseIterDictIndexer(TerrierIndexer, Indexer):
 
     def _filter_iterable(self, it, indexed_fields):
         # Only include necessary fields: those that are indexed, metadata fields, and docno
-        
+        # Also, check that the provided iterator is a suitable format
+
         if self.pretokenised:
             all_fields = {'docno', "toks"} | set(self.meta.keys())
         else:
             all_fields = {'docno'} | set(indexed_fields) | set(self.meta.keys())
-        for doc in it:
-            yield {f: doc[f] for f in all_fields}
+
+        (first_doc,), it = more_itertools.spy(it) # peek at the first document and validate it
+        self._validate_doc_dict(first_doc)
+
+        # important: return an iterator here, rather than make this function a generator,
+        # to be sure that the validation above happens when _filter_iterable is called,
+        # rather than on the first invocation of next()
+        return ({f: doc[f] for f in all_fields} for doc in it)
+
+    def _is_dict(self, obj):
+        return hasattr(obj, '__getitem__') and hasattr(obj, 'items')
+
+    def _validate_doc_dict(self, obj):
+        """
+        Raise errors/warnings for common indexing mistakes
+        """
+        if not self._is_dict(obj):
+            raise ValueError("Was passed %s while expected dict-like object" % (str(type(obj))))
+        if self.meta is not None:
+            for k in self.meta:
+                if k not in obj:
+                    raise ValueError(f"Indexing meta key {k} not found in first document (keys {list(obj.keys())})")
+                if len(obj) > int(self.meta[k]):
+                    msg = f"Indexing meta key {k} length requested {self.meta[k]} but exceeded in first document (actual length {len(obj[k])}). " + \
+                          f"Increase the length in the meta dict for the indexer, e.g., pt.IterDictIndexer(..., meta={ {k: len(obj[k])} })."
+                    if k == 'docno':
+                        # docnos that are truncated will cause major issues; raise an error
+                        raise ValueError(msg)
+                    else:
+                        # Other fields may not matter as much; just show a warning
+                        warn(msg)
 
 
 class _IterDictIndexer_nofifo(_BaseIterDictIndexer):
@@ -849,7 +886,7 @@ class _IterDictIndexer_nofifo(_BaseIterDictIndexer):
             #     {'docno' : 'd1', 'toks' : {'a' : 1, 'aa' : 2}}
             # ]
             
-            iter_docs = DocListIterator(it)
+            iter_docs = DocListIterator(self._filter_iterable(it, fields))
             self.index_called = True
             index.indexDocuments(iter_docs)
             iter_docs = None
